// -*-C++-*-
//
// File:      OmSourceRegions.h
// Author:    Jorgo Bakker (jbakker@astro.estec.esa.nl)
// Generated: Tue Sep 10 10:58:50 CEST 2002
// Usage:     interface of C++/f90
// Info:      
#include "OmSourceRegions.h"
#include "DopeVector.h"
#include "OmImageDisplay.h"
#include <cmath>
#include <iostream>




OmSourceRegions* OmImageRegions::_oldRegions = 0;
OmSourceRegions* OmImageRegions::_backgroundRegions = 0;

int OmImageRegions::_noBackgroundRegions = 0;
int OmImageRegions::_noRegions = 0;
string OmImageRegions::_colour = "yellow";
/****************************************************************
*
***************************************************************/
OmSourceRegions* OmImageRegions::GetRegions(OmImageDisplay*  display) 
{
   
 
    GeoRegion gReg = display->regionSelection(ImageDisplayer::IMAGE, _colour);


    //        cout << "Omsourceregions.cc Getregions line 36\n"; 
 
    // count the number of selected points
    int nPoints = 0;
   
    
    for(vector<Component>::const_iterator it = gReg.begin();
	it != gReg.end(); it++)
    {  
        nPoints += (*it).size();
     
    }
    


    //        cout << "Getregions line 49 - nPoints=" << nPoints << endl; 
  
    if (nPoints < 1)
    {
	     
        return  0;


    }


    // now we contruct a dopevector...
    OmSourceRegions* points = new OmSourceRegions(1, nPoints);

    unsigned long idx = 1;
 
  
    for(vector<Component>::const_iterator it = gReg.begin();
	it != gReg.end(); it++) { // should be one only....
        const Component& component = (*it);
        for(vector<FShape*>::const_iterator ic = component.begin();
	    ic != component.end(); ic++) 
	  {

	      (*points)[idx++] = (*ic)->refPoint();
	  }
    }
 
  
    return points;    // The selected points  
 
}  


/**********************************************************************
 *
 *********************************************************************/
void OmImageRegions::deleteAllRegions( ) 
{
     
    delete( _oldRegions );
    _noRegions = 0;
    _oldRegions = 0;
}

/**********************************************************************
 *
 *********************************************************************/
void OmImageRegions::deleteRegion(int no, OmImageDisplay*  _display) 
{
     
    // Get all the regions from the image
    OmSourceRegions* regions = GetRegions( _display );
   
    if (regions == 0) return;

    
    int noPoints = regions->ubound(); 
   
    if ( _oldRegions) 
        delete ( _oldRegions );       
   

    if ( noPoints == 1)  // Deleting last region
    {

      _noRegions = 0;
      _oldRegions = 0;
 
      return;



    }
    _oldRegions = new OmSourceRegions(1, noPoints - 1); // One fewer regions

  
    int k = _oldRegions->lbound();

    _noRegions = 0; 
    for(int i = regions->lbound(); i <= regions->ubound(); i++)  
    {
		   
        if ( i != no)
	{   
	    (*_oldRegions)[k++] = (*regions)[i];
   
	    _noRegions++;
        }
    }

}

/**********************************************************************
 *
 *********************************************************************/
OmSourceRegions* OmImageRegions::GetNewRegions(OmImageDisplay*  display, float nx, float ny) 
{
     

  

    // Get all the regions from the image
    OmSourceRegions* regions = GetRegions( display );

    if (regions == 0) return 0; // No regions
  
    int noPoints = regions->ubound();  // The number of regions



   


    if ( _oldRegions == 0 ) // No previous regions
    {


        _oldRegions = new OmSourceRegions( regions );

	_noRegions = noPoints;
        
        return regions;

    }




    // Compare the regions with the old ones to see if any new ones
    float tolerance = 5.0;
  
  
    int *keep = new int[noPoints + 1];
    int nToKeep = noPoints; 
  
    // populate the 'keep' and count the unique points
   
    for (int i = regions->lbound(); i <= regions->ubound(); i++)
    {
        keep[i] = true;

        float x = (*regions)[i].x(); 
        float y = (*regions)[i].y(); 
        if (x < 0.0 || x > nx || y < 0.0 || y > ny)
	{
             
              keep[i] = false;
	      nToKeep = nToKeep - 1;
	      continue;


        }

	for(int j = _oldRegions->lbound(); j <= _oldRegions->ubound(); j++)  
	{
		   
	  float dx = (*regions)[i].x() - (*_oldRegions)[j].x();
           
	  float dy = (*regions)[i].y() - (*_oldRegions)[j].y();
	  float radius = sqrt(dx * dx + dy * dy);
         
            
	  if (radius <= tolerance)
	  {
	      keep[i] = false;
	      nToKeep = nToKeep - 1;
	      break;
	  }
	}
    }





 

    if (nToKeep ==  0) // No new regions selected 
    {

        delete( regions );
        regions = 0;
        delete [] keep;
                 
        return  0;
 
    }


 


    // now we create an index of all regions that are new
    
    OmSourceRegions* tempPoints = new OmSourceRegions(1, nToKeep); // Stores the newly selected regions





        
    int j = 1;
   

    for(int i = regions->lbound(); i <= regions->ubound(); i++)  
    {
        
        if (keep[i]) 
        {
	  
	    (*tempPoints)[j++] = (*regions)[i];

	}
    }


    delete [] keep; 

    
    // Replace the old regions with the new ones
   
    if ( _oldRegions ) delete(_oldRegions );
    _oldRegions = new OmSourceRegions( regions );


    delete(regions); 
    regions = 0;

    _noRegions = noPoints;

    

    return tempPoints;  // The selected points  
 
}  
/************************************************************************
 *
 ***********************************************************************/
void  OmImageRegions::updateBackgroundRegions(OmSourceRegions *regions)
{



  //    cout << "Update background regions line 360\n";

 
  
 
  //  cout << "updatebackgroundregions line 262\n";
    if (regions == 0) return;



    int size1 = regions->ubound();


    //    cout << "Update background regions line 436 size1 = " << size1 << endl;

 
    if (size1 <= 0) return;
   
   



    //  cout << "line 457 nobackgroundregions=" << _noBackgroundRegions << endl;     

    if ( _noBackgroundRegions <= 0)  // No existing background regions
    {
       
      _backgroundRegions = new OmSourceRegions( regions );

      
     _noBackgroundRegions = size1;      


     //   cout << "updatebackgroundregions line 464 lbound background regions=" << _backgroundRegions->lbound() << endl;
     //  cout << "updatebackgroundregions line 464 ubound background regions=" << _backgroundRegions->ubound() << endl;

     //  cout << "nobackgroundregions=" << _noBackgroundRegions << endl;     
       return;

    }


    int size2 = _backgroundRegions->ubound();
 

    //   cout << "updatebackgroundregions line 387 size2=" << size2 << endl;


              
    int size3 = size1 + size2;
    OmSourceRegions* tempRegions = new OmSourceRegions( 1, size3 );





    //    cout << "updatebackgroundregions line 398\n";




    for (int i =  _backgroundRegions->lbound(); i <=  _backgroundRegions->ubound(); i++)
    {
        (*tempRegions)[i] = (*_backgroundRegions)[i];

    }


    //    cout << "updatebackgroundregions line 303 nobackgroundregions= " << _noBackgroundRegions << endl;;



    for (int i =  regions->lbound(); i <=  regions->ubound(); i++)
    {
        (*tempRegions)[i + size2] =   (*regions)[i];

    }




    delete( _backgroundRegions );
    _backgroundRegions = 0;

    _backgroundRegions = new OmSourceRegions( tempRegions );
    _noBackgroundRegions = size3;      
  
    delete( tempRegions );


    //    cout << "Line 435 updatebackgroundregions the new background regions\n";

    //    cout << "Line 435 updatebackgroundregions nobackgroundregions=" << _noBackgroundRegions << endl;


    //    for (int i =  _backgroundRegions->lbound(); i <=  _backgroundRegions->ubound(); i++)
    //    {
      
    //        float x = (*_backgroundRegions)[i].x(); 
    //        float y = (*_backgroundRegions)[i].y(); 
	//	    cout << i << ") x=" << x << " y=" << y << endl;      

    //    }


    //    cout << "updatebackgroundregions line 436 nobackgroundregions=" << _noBackgroundRegions << endl;



    //   cout << "updatebackgroundregions line 533 lbound background regions=" << _backgroundRegions->lbound() << endl;
    //  cout << "updatebackgroundregions line 533 ubound background regions=" << _backgroundRegions->ubound() << endl;



    //    cout << "updatebackgroundregions line 543 nobackgroundregions=" << _noBackgroundRegions << 
    //      " ubound backgroundregions=" << _backgroundRegions->ubound() << endl;





}

/**********************************************************************
 *
 *********************************************************************/
void OmImageRegions::deleteBackgroundRegions(OmImageDisplay*  _display) 
{

  //  if (_noBackgroundRegions == 0) return;

  //    cout << "deleteBackgroundRegions line 247\n";

    // Get all the regions from the image
    OmSourceRegions* regions = GetRegions(_display);


    if (regions == 0) 
    {
	//	cout << "Returning regions ==0\n";

        return;
 
    }
    //  cout << "deleteBackgroundRegions line 253 \n";



    int noPoints = regions->ubound(); 
   
    if (noPoints <= 0) return;

    // Compare the regions with the old ones
    float tolerance = 0.5;
  


    // Need to remove the background regions from the combined reguions list
  
    // populate the 'keep' and count the unique points
   

    int *keep = new int[noPoints + 1];
    int nToKeep = noPoints; 

    //  cout << "deleteBackgroundRegions line 274 \n";

    for (int i = regions->lbound(); i <= regions->ubound(); i++)
    {
       
      //      cout << "Line 277 i=" << i << endl;
	for(int j = _backgroundRegions->lbound(); j <= _backgroundRegions->ubound(); j++)  
	{
		   
	  float dx = (*_backgroundRegions)[j].x() - (*regions)[i].x();
           
	  float dy = (*_backgroundRegions)[j].y() - (*regions)[i].y();
	  float radius = sqrt(dx * dx + dy * dy);
         
            
	  if (radius <= tolerance)
	  {
	    //	    cout << "Line 289 discarding region " << j << endl;
	      keep[i] = false;
	      nToKeep = nToKeep - 1;
	      break;
	  }
	}
    }
    //  cout << "deleteBackgroundRegions line 298 \n";



    if (nToKeep ==  0) 
    {
      delete [] keep;
      return;
	
 
    }  

    //cout << "deleteBackgroundRegions line 310 \n";



    // now we create an index of all regions that are new

        
    OmSourceRegions* tempPoints = new OmSourceRegions(1, nToKeep);
  
     
    int j = 1;
   
    for(int i = regions->lbound(); i <= regions->ubound(); i++)  
    {
        
        if (keep[i]) 
        {
	  
	    (*tempPoints)[j++] = (*regions)[i];

	}
    }
    delete [] keep; 
    
    //  cout << "deleteBackgroundRegions line 334 \n";

    // Replace the old regions with the new ones
   
    if (_noRegions > 0) delete(_oldRegions);
    _oldRegions = new OmSourceRegions(tempPoints);
  
    delete(tempPoints); 
    _noRegions = noPoints;

    delete(OmImageRegions::_backgroundRegions);
    OmImageRegions::_backgroundRegions = 0;
    delete(regions); 
    regions = 0;
 
    //   cout << "deleteBackgroundRegions line 347 \n";


}  



